#! usr/bin/env python3

'''
This script is used to stat the gene features based on gff file, 
including gene length, exon length, first exon length,
exon number,  and intron length.
'''

import getopt
import sys

opts, args = getopt.getopt(sys.argv[1:],'-i:-o:-h',['inputfile=','ouputfile=','help'])

def usage():
	print('usage : python3 script -i inputfile -o outputfile -h help infor')

for opt_name,opt_value in opts:
	if opt_name in ('-h','--help'):
		usage()
		sys.exit()
	if opt_name in ('-i','--inputfile'):
		inputfile = opt_value
	if opt_name in ('-o','--outputfile'):
		outputfile = opt_value

# python3 00.stat_gene_feature.py -i gff -o stat

def seqfile2dict_func(seqfile):
	in_f = open (seqfile,'r')
	for lin in in_f:
		print(lin)

def inf_func(inputfile):
	inf = open (inputfile,'r')
	arrs = []
	for lin in inf:
		if "#" not in lin:
			arrs.append(lin.strip())
	return arrs

def gene_length_func(inf_arrs):
	dict = {}
	for inf_arr in inf_arrs:
		if inf_arr.split('\t')[2] == "gene":
			gen_len =  int(inf_arr.split('\t')[4]) - int(inf_arr.split('\t')[3]) + 1
			dict[inf_arr.split('\t')[8].replace(';','').replace('ID=','')] = gen_len
	return dict

def mRNA_length_func(inf_arrs):
	dict = {}
	for inf_arr in inf_arrs:
		if inf_arr.split('\t')[2] == "mRNA":
			mRNA_len =  int(inf_arr.split('\t')[4]) - int(inf_arr.split('\t')[3]) + 1
			dict[inf_arr.strip().split('\t')[8].split(';')[1].replace('Parent=','')] = mRNA_len
	return dict

def exon_number_len_func(inf_ars):
	dict_exon_num = {}
	dict_exon_len = {}
	for inf_arr in inf_arrs:
		if inf_arr.split('\t')[2] == "exon":
			gene_id = inf_arr.split('\t')[8].split(';')[0].replace('Parent=','').split('.')[0]
			exon_len =  int(inf_arr.split('\t')[4]) - int(inf_arr.split('\t')[3]) + 1
			if gene_id not in dict_exon_num.keys() and gene_id not in dict_exon_len.keys():
				dict_exon_num[gene_id] = 1
				dict_exon_len[gene_id] = exon_len 
			else:
				dict_exon_num[gene_id] += 1
				dict_exon_len[gene_id] += exon_len
	return dict_exon_num, dict_exon_len

def intron_length_func(dict_mRNA_len,dict_exon_len):
	dict_intron_len = {}
	for k in dict_mRNA_len.keys():
		if k in dict_exon_len.keys():
			dict_intron_len[k] = dict_mRNA_len[k] - dict_exon_len[k]
	return dict_intron_len	

def first_exon_length_func(inf_arrs):
	dict_first_exon_len = {}
	for inf_arr in inf_arrs:
		if inf_arr.split('\t')[2] == "exon":
			gene_id = inf_arr.split('\t')[8].split(';')[0].split('.')[0].replace('Parent=','')
			exon_len =  int(inf_arr.split('\t')[4]) - int(inf_arr.split('\t')[3]) + 1 
			if  inf_arr.split('\t')[6] == "+":
				if gene_id not in dict_first_exon_len.keys():
					dict_first_exon_len[gene_id] = exon_len
			else:
				dict_first_exon_len[gene_id] = exon_len
	return dict_first_exon_len
				 
	
def com_func(dict_gen_len,dict_mRNA_len,dict_exon_num,dict_exon_len,dict_intron_len,dict_first_exon_len,outputfile):
	outf = open(outputfile, 'w')
	outf.write("Gene_ID\tGene_len\tmRNA_len\tExon_len\tExon_num\tFirst_exon_len\tIntron_len\n")
	for k in dict_gen_len.keys():
		gen_len = dict_gen_len[k]
		if k in dict_mRNA_len.keys():
			mRNA_len = dict_mRNA_len[k]
		else:
			mRNA_len = "NA"

		if k in dict_exon_len.keys():
			exon_len = dict_exon_len[k]
		else:
			exon_len = "NA"

		if k in dict_exon_num.keys():
			exon_num = dict_exon_num[k]
		else:
			exon_num = "NA"

		if k in dict_first_exon_len.keys():
			first_exon_len = dict_first_exon_len[k]
		else:
			first_exon_len = "NA"

		if k in dict_intron_len.keys():
			intron_len = dict_intron_len[k]
		else:
			intron_len = "NA"
		outinfor = k + '\t' + str(gen_len) + '\t' + str(mRNA_len) + '\t' +str(exon_len) + '\t' +str(exon_num) + '\t' + str(first_exon_len) + '\t' + str(intron_len) + '\n'
		outf.write(outinfor)


if __name__ == '__main__':
	inf_arrs = inf_func(inputfile)
	dict_gen_len = gene_length_func(inf_arrs) 
	dict_mRNA_len = mRNA_length_func(inf_arrs) 
	dict_exon_num, dict_exon_len = exon_number_len_func(inf_arrs)
	dict_intron_len = intron_length_func(dict_mRNA_len,dict_exon_len)
	dict_first_exon_len =first_exon_length_func(inf_arrs)
	com_func(dict_gen_len,dict_mRNA_len,dict_exon_num,dict_exon_len,dict_intron_len,dict_first_exon_len,outputfile)
